@@ -125,7 +125,9 @@ static void parse_url(const gchar * url, gchar ** user, gchar ** pass, gchar **
 
 	if (!strncasecmp("http://", ptr, 7))
 		ptr += 7;
-	if ((h = strchr(ptr, '@')))
+	h = strchr(ptr, '@');
+	f = strchr(ptr, '/');
+	if (h != NULL && (!f || h < f))
 	{
 		*h = '\0';
 		p = strchr(ptr, ':');
@@ -148,7 +150,6 @@ static void parse_url(const gchar * url, gchar ** user, gchar ** pass, gchar **
 		h = ptr;
 	}
 	pt = strchr(ptr, ':');
-	f = strchr(ptr, '/');
 	if (pt != NULL && (f == NULL || pt < f))
 	{
 		*pt = '\0';
@@ -204,7 +205,7 @@ static void show_error_message(gchar *error)
 	if(!error_dialog)
 	{
 		GDK_THREADS_ENTER();
-		error_dialog = xmms_show_message(_("Error"), error, _("Ok"), FALSE,
+		error_dialog = xmms_show_message(_("Error"), error, _("OK"), FALSE,
 						 NULL, NULL);
 		gtk_signal_connect(GTK_OBJECT(error_dialog),
 				   "destroy",
@@ -217,7 +218,7 @@ static void show_error_message(gchar *error)
 int mpg123_http_read(gpointer data, gint length)
 {
 	gint len, cnt, off = 0, meta_len, meta_off = 0, i;
-	gchar *meta_data, **tags, *temp, *title;
+	char *meta_data, **tags;
 
 	http_wait_for_data(length);
 
@@ -251,8 +252,8 @@ int mpg123_http_read(gpointer data, gint length)
 					{
 						if (!strncasecmp(tags[i], "StreamTitle=", 12))
 						{
-							temp = g_strdup(tags[i] + 13);
-							title = g_strdup_printf("%s (%s)", temp, icy_name);
+							char *temp = tags[i] + 13;
+							char *title = g_strdup_printf("%s (%s)", temp, icy_name);
 							mpg123_ip.set_info(title, -1, mpg123_bitrate * 1000, mpg123_frequency, mpg123_stereo);
 							g_free(title);
 						}
@@ -263,6 +264,8 @@ int mpg123_http_read(gpointer data, gint length)
 				}
 				g_free(meta_data);
 			}
+			if (!http_used())
+				http_wait_for_data(length - off);
 			cnt = min3(len, buffer_length - rd_index, http_used());
 		}
 		else if ((icy_metaint > 0) && (mpg123_cfg.cast_title_streaming))
@@ -320,7 +323,7 @@ gint mpg123_http_read_line(gchar * buf, gint size)
 	return i;
 }
 
-void *http_buffer_loop(void *arg)
+static void *http_buffer_loop(void *arg)
 {
 	gchar line[1024], *user, *pass, *host, *filename,
 	     *status, *url, *temp, *file;
@@ -329,8 +332,13 @@ void *http_buffer_loop(void *arg)
 	gboolean redirect;
 	int udp_sock = 0;
 	fd_set set;
+#ifdef USE_IPV6
+	struct addrinfo hints, *res, *res0;
+	char service[6];
+#else
 	struct hostent *hp;
 	struct sockaddr_in address;
+#endif
 	struct timeval tv;
 
 	url = (gchar *) arg;
@@ -352,6 +360,45 @@ void *http_buffer_loop(void *arg)
 		chost = mpg123_cfg.use_proxy ? mpg123_cfg.proxy_host : host;
 		cport = mpg123_cfg.use_proxy ? mpg123_cfg.proxy_port : port;
 
+#ifdef USE_IPV6
+		g_snprintf(service, 6, "%d", cport);
+		memset(&hints, 0, sizeof(hints));
+		hints.ai_socktype = SOCK_STREAM;
+		if (! getaddrinfo(chost, service, &hints, &res0)) {
+			eof = TRUE;
+			for (res = res0; res; res = res->ai_next) {
+				if ((sock = socket (res->ai_family, res->ai_socktype, res->ai_protocol)) < 0)
+					continue;
+				fcntl(sock, F_SETFL, O_NONBLOCK);
+				status = g_strdup_printf(_("CONNECTING TO %s:%d"), chost, cport);
+				mpg123_ip.set_info_text(status);
+				g_free(status);
+				((struct sockaddr_in6 *)res->ai_addr)->sin6_port = htons(cport);
+				if (connect(sock, res->ai_addr, res->ai_addrlen) < 0) {
+					if (errno != EINPROGRESS) {
+						close(sock);
+						continue;
+					}
+				}
+				eof = FALSE;
+				break;
+			}
+			freeaddrinfo(res0);
+			if (eof) {
+				status = g_strdup_printf(_("Couldn't connect to host %s:%d"), chost, cport);
+				show_error_message(status);
+				g_free(status);
+				mpg123_ip.set_info_text(NULL);
+			}
+		} else {
+			status = g_strdup_printf(_("Couldn't look up host %s"), chost);
+			show_error_message(status);
+			g_free(status);
+
+			mpg123_ip.set_info_text(NULL);
+			eof = TRUE;
+		}
+#else
 		sock = socket(AF_INET, SOCK_STREAM, 0);
 		fcntl(sock, F_SETFL, O_NONBLOCK);
 		address.sin_family = AF_INET;
@@ -369,9 +416,11 @@ void *http_buffer_loop(void *arg)
 			mpg123_ip.set_info_text(NULL);
 			eof = TRUE;
 		}
+#endif
 
 		if (!eof)
 		{
+#ifndef USE_IPV6
 			memcpy(&address.sin_addr.s_addr, *(hp->h_addr_list), sizeof (address.sin_addr.s_addr));
 			address.sin_port = g_htons(cport);
 
@@ -390,6 +439,7 @@ void *http_buffer_loop(void *arg)
 					eof = TRUE;
 				}
 			}
+#endif
 			while (going)
 			{
 				tv.tv_sec = 0;
@@ -552,7 +602,10 @@ void *http_buffer_loop(void *arg)
 		if(redirect)
 		{
 			if (output_file)
+			{
 				fclose(output_file);
+				output_file = NULL;
+			}
 			close(sock);
 			g_free(user);
 			g_free(pass);
@@ -563,18 +616,28 @@ void *http_buffer_loop(void *arg)
 	
 	if (mpg123_cfg.save_http_stream)
 	{
-		gchar *output_name;
+		char *output_name, *fname, *temp;
+		int i = 1;
+
 		file = mpg123_http_get_title(url);
-		output_name = file;
-		if (!strncasecmp(output_name, "http://", 7))
-			output_name += 7;
-		temp = strrchr(output_name, '.');
+		fname = file;
+		if (!strncasecmp(fname, "http://", 7))
+			fname += 7;
+		temp = strrchr(fname, '.');
 		if (temp && !strcasecmp(temp, ".mp3"))
 			*temp = '\0';
 
-		while ((temp = strchr(output_name, '/')))
+		while ((temp = strchr(fname, '/')))
 			*temp = '_';
-		output_name = g_strdup_printf("%s/%s.mp3", mpg123_cfg.save_http_path, output_name);
+		output_name = g_strdup_printf("%s/%s.mp3",
+					      mpg123_cfg.save_http_path, fname);
+		while (!access(output_name, F_OK) && i < 100000)
+		{
+			g_free(output_name);
+			output_name = g_strdup_printf("%s/%s-%d.mp3",
+						      mpg123_cfg.save_http_path,
+						      fname, i++);
+		}
 
 		g_free(file);
 
@@ -632,13 +695,15 @@ void *http_buffer_loop(void *arg)
 		if (mpg123_cfg.use_udp_channel && udp_sock != 0)
 			if (udp_check_for_data(udp_sock) < 0)
 			{
-				g_message("closing udp");
 				close(udp_sock);
 				udp_sock = 0;
 			}
 	}
 	if (output_file)
+	{
 		fclose(output_file);
+		output_file = NULL;
+	}
 	close(sock);
 	if (udp_sock != 0)
 		close(udp_sock);
@@ -675,35 +740,51 @@ int mpg123_http_open(gchar * _url)
 	return 0;
 }
 
-gchar *mpg123_http_get_title(gchar * url)
+char *mpg123_http_get_title(char *url)
 {
 	if (icy_name)
 		return g_strdup(icy_name);
+	if (g_basename(url) && strlen(g_basename(url)) > 0)
+		return g_strdup(g_basename(url));
 	return g_strdup(url);
 }
 
 /* Start UDP Channel specific stuff */
 
 /* Find a good local udp port and bind udp_sock to it, return the port */
-static int udp_establish_listener (gint *sock)
+static int udp_establish_listener(int *sock)
 {
+#ifdef USE_IPV6
+	struct sockaddr_in6 sin;
+	socklen_t sinlen = sizeof (struct sockaddr_in6);
+#else
 	struct sockaddr_in sin;
 	socklen_t sinlen = sizeof (struct sockaddr_in);
+#endif
 	
 #ifdef DEBUG_UDP
 	fprintf (stderr,"Establishing udp listener\n");
 #endif
 	
+#ifdef USE_IPV6
+	if ((*sock = socket(AF_INET6, SOCK_DGRAM, 0)) < 0)
+#else
 	if ((*sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
+#endif
 	{
 		g_log(NULL, G_LOG_LEVEL_CRITICAL,
-		      "udp_establish_listener(): unable to create socket");
+		      "udp_establish_listener(): unable to create socket: %s",
+		      strerror(errno));
 		return -1;
 	}
 
 	memset(&sin, 0, sinlen);
+#ifdef USE_IPV6
+	sin.sin6_family = AF_INET6;
+#else
 	sin.sin_family = AF_INET;
 	sin.sin_addr.s_addr = g_htonl(INADDR_ANY);
+#endif
 			
 	if (bind(*sock, (struct sockaddr *)&sin, sinlen) < 0)
 	{
@@ -733,7 +814,11 @@ static int udp_establish_listener (gint *sock)
 	fprintf (stderr,"Listening on local %s:%d\n", inet_ntoa(sin.sin_addr), g_ntohs(sin.sin_port));
 #endif
 	
+#ifdef USE_IPV6
+	return g_ntohs(sin.sin6_port);
+#else
 	return g_ntohs(sin.sin_port);
+#endif
 }
 
 static int udp_check_for_data(int sock)
@@ -742,10 +827,14 @@ static int udp_check_for_data(int sock)
 	char *valptr;
 	gchar *title;
 	gint len, i;
+#ifdef USE_IPV6
+	struct sockaddr_in6 from;
+#else
 	struct sockaddr_in from;
+#endif
 	socklen_t fromlen;
 
-	fromlen = sizeof(struct sockaddr_in);
+	fromlen = sizeof(from);
 	
 	if ((len = recvfrom(sock, buf, 1024, 0, (struct sockaddr *)&from, &fromlen)) < 0)
 	{
@@ -801,8 +890,9 @@ static int udp_check_for_data(int sock)
 		else if (strstr(lines[i], "x-audiocast-streammsg") != NULL)
 		{
 			/*  mpg123_ip.set_info(title, -1, mpg123_bitrate * 1000, mpg123_frequency, mpg123_stereo); */
-			xmms_show_message(_("Message"), valptr, _("Ok"),
-					  FALSE, NULL, NULL);
+/*  			xmms_show_message(_("Message"), valptr, _("OK"), */
+/*  					  FALSE, NULL, NULL); */
+			g_message("Stream_message: %s", valptr);
 		}
 
 #if 0
@@ -812,9 +902,8 @@ static int udp_check_for_data(int sock)
 			if (lasturl && g_strcmp (valptr, lasturl))
 			{
 				c_message (stderr, "Song URL: %s\n", valptr);
-				if (lasturl)
-					free (lasturl);
-				lasturl = strdup (valptr);
+				g_free (lasturl);
+				lasturl = g_strdup (valptr);
 			}
 		}
 #endif
@@ -824,13 +913,20 @@ static int udp_check_for_data(int sock)
 			sprintf(obuf, "x-audiocast-ack: %ld \r\n", atol(valptr));
 			if (sendto(sock, obuf, strlen(obuf), 0, (struct sockaddr *) &from, fromlen) < 0)
 			{
-				g_log(NULL, G_LOG_LEVEL_CRITICAL,
+				g_log(NULL, G_LOG_LEVEL_WARNING,
 				      "udp_check_for_data(): Unable to send ack to server: %s", strerror(errno));
 			}
 #ifdef DEBUG_UDP
 			else
 				fprintf(stderr,"Sent ack: %s", obuf);
+#ifdef USE_IPV6
+{
+			char adr[INET6_ADDRSTRLEN];
+			inet_ntop(AF_INET6, &from.sin6_addr, adr, INET6_ADDRSTRLEN);
+			fprintf (stderr,"Remote: [%s]:%d\n", adr, g_ntohs(from.sin6_port));
+#else
 			fprintf (stderr,"Remote: %s:%d\n", inet_ntoa(from.sin_addr), g_ntohs(from.sin_port));
+#endif
 #endif
 		}
 	}
