@@ -8,7 +8,7 @@
  * Author:        Espen Skoglund <espensk@stud.cs.uit.no>
  * Created at:    Fri Feb  5 23:55:13 1999
  *                
- * $Id: id3.c,v 1.11 2002/03/04 20:21:34 havard Exp $
+ * $Id: id3.c,v 1.13 2002/09/27 23:39:33 havard Exp $
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -41,7 +41,6 @@
 #include "id3_header.h"
 
 
-
 /*
 **
 ** Functions for accessing the ID3 tag using a memory pointer.
@@ -55,7 +54,7 @@
  *    upon success, or -1 if an error occured.
  *
  */
-static int id3_seek_mem(id3_t *id3, int offset)
+static int id3_seek_mem(struct id3_tag *id3, int offset)
 {
     if (id3->id3_pos + offset > id3->id3_tagsize ||
 	id3->id3_pos + offset < 0)
@@ -78,7 +77,7 @@ static int id3_seek_mem(id3_t *id3, int offset)
  *    read, or NULL upon error.
  *
  */
-static void *id3_read_mem(id3_t *id3, void *buf, int size)
+static void *id3_read_mem(struct id3_tag *id3, void *buf, int size)
 {
     void *ret = id3->s.me.id3_ptr;
 
@@ -122,7 +121,7 @@ static void *id3_read_mem(id3_t *id3, void *buf, int size)
  *    upon success, or -1 if an error occured.
  *
  */
-static int id3_seek_fd(id3_t *id3, int offset)
+static int id3_seek_fd(struct id3_tag *id3, int offset)
 {
     /*
      * Check boundary.
@@ -149,7 +148,7 @@ static int id3_seek_fd(id3_t *id3, int offset)
  *    read, or NULL upon error.
  *
  */
-static void *id3_read_fd(id3_t *id3, void *buf, int size)
+static void *id3_read_fd(struct id3_tag *id3, void *buf, int size)
 {
     int done = 0;
 
@@ -207,7 +206,7 @@ static void *id3_read_fd(id3_t *id3, void *buf, int size)
  *    upon success, or -1 if an error occured.
  *
  */
-static int id3_seek_fp(id3_t *id3, int offset)
+static int id3_seek_fp(struct id3_tag *id3, int offset)
 {
     /*
      * Check boundary.
@@ -257,7 +256,7 @@ static int id3_seek_fp(id3_t *id3, int offset)
  *    read, or NULL upon error.
  *
  */
-static void *id3_read_fp(id3_t *id3, void *buf, int size)
+static void *id3_read_fp(struct id3_tag *id3, void *buf, int size)
 {
     int ret;
 
@@ -302,14 +301,14 @@ static void *id3_read_fp(id3_t *id3, void *buf, int size)
  *    structure describing the ID3 tag, or NULL if an error occured.
  *
  */
-id3_t *id3_open_mem(void *ptr, int flags)
+struct id3_tag *id3_open_mem(void *ptr, int flags)
 {
-    id3_t *id3;
+    struct id3_tag *id3;
 
     /*
      * Allocate ID3 structure.
      */
-    id3 = g_malloc0( sizeof(id3_t) );
+    id3 = g_malloc0( sizeof(struct id3_tag) );
 
     /*
      * Initialize access pointers.
@@ -346,14 +345,14 @@ id3_t *id3_open_mem(void *ptr, int flags)
  *    structure describing the ID3 tag, or NULL if an error occured.
  *
  */
-id3_t *id3_open_fd(int fd, int flags)
+struct id3_tag *id3_open_fd(int fd, int flags)
 {
-    id3_t *id3;
+    struct id3_tag *id3;
 
     /*
      * Allocate ID3 structure.
      */
-    id3 = g_malloc0( sizeof(id3_t) );
+    id3 = g_malloc0( sizeof(struct id3_tag) );
 
     /*
      * Initialize access pointers.
@@ -399,14 +398,14 @@ id3_t *id3_open_fd(int fd, int flags)
  *    structure describing the ID3 tag, or NULL if an error occured.
  *
  */
-id3_t *id3_open_fp(FILE *fp, int flags)
+struct id3_tag *id3_open_fp(FILE *fp, int flags)
 {
-    id3_t *id3;
+    struct id3_tag *id3;
 
     /*
      * Allocate ID3 structure.
      */
-    id3 = g_malloc0(sizeof(id3_t));
+    id3 = g_malloc0(sizeof(struct id3_tag));
 
     /*
      * Initialize access pointers.
@@ -452,7 +451,7 @@ id3_t *id3_open_fp(FILE *fp, int flags)
  *    Free all resources assoicated with the ID3 tag.
  *
  */
-int id3_close(id3_t *id3)
+int id3_close(struct id3_tag *id3)
 {
     int ret = 0;
     
@@ -485,7 +484,8 @@ int id3_close(id3_t *id3)
  *    directly after the tag.
  *
  */
-int id3_tell(id3_t *id3)
+#if 0
+int id3_tell(struct id3_tag *id3)
 {
     if ( id3->id3_newtag ) {
 	return 0;
@@ -493,6 +493,7 @@ int id3_tell(id3_t *id3)
 	return id3->id3_tagsize + 3 + sizeof(id3_taghdr_t);
     }
 }
+#endif
 
 
 /*
@@ -504,7 +505,7 @@ int id3_tell(id3_t *id3)
  *    whenever the file is altered.
  *
  */
-int id3_alter_file(id3_t *id3)
+int id3_alter_file(struct id3_tag *id3)
 {
     /*
      * List of frame classes that should be discarded whenever the
@@ -515,7 +516,7 @@ int id3_alter_file(id3_t *id3)
 	ID3_SYTC, ID3_RVAD, ID3_TENC, ID3_TLEN, ID3_TSIZ,
 	0
     };
-    id3_frame_t *fr;
+    struct id3_frame *fr;
     guint32 id, i = 0;
 
     /*
@@ -564,61 +565,68 @@ static int safe_write(int fd, void *buf, int size)
  *    upon success, or -1 if an error occured.
  *
  */
-int id3_write_tag(id3_t *id3, int fd)
+int id3_write_tag(struct id3_tag *id3, int fd)
 {
-    id3_frame_t *fr;
-    id3_taghdr_t taghdr;
-    GList *node;
-    int size = 0;
-
-    /*
-     * Calculate size of ID3 tag.
-     */
-    for (node = id3->id3_frame; node != NULL; node = node->next) {
-	fr = node->data;
-	size += fr->fr_size + sizeof(id3_framehdr_t);
-    }
-
-    /*
-     * Write tag header.
-     */
-    taghdr.th_version = id3->id3_version;
-    taghdr.th_revision = id3->id3_revision;
-    taghdr.th_flags = id3->id3_flags;
-    taghdr.th_size = g_htonl( ID3_SET_SIZE28(size) );
-
-    if ( safe_write(fd, "ID3", 3) == -1 )
-	return -1;
-    if ( safe_write(fd, &taghdr, sizeof(taghdr)) == -1 )
-	return -1;
-
-    /*
-     * TODO: Write extended header.
-     */
-#if 0
-    if ( id3->id3_flags & ID3_THFLAG_EXT ) {
-	id3_exthdr_t exthdr;
-    }
-#endif
-	    
-    for (node = id3->id3_frame; node != NULL; node = node->next) {
-	id3_framehdr_t fhdr;
-
-	fr = node->data;
+	struct id3_frame *fr;
+	GList *node;
+	int size = 0;
+	char buf[ID3_TAGHDR_SIZE];
 
 	/*
-	 * TODO: Support compressed headers, encoded
-	 * headers, and grouping info.
+	 * Calculate size of ID3 tag.
 	 */
-	fhdr.fh_id = fr->fr_desc ? g_htonl(fr->fr_desc->fd_id) : 0;
-	fhdr.fh_size = g_htonl( fr->fr_size );
-	fhdr.fh_flags = g_ntohl( fr->fr_flags );
+	for (node = id3->id3_frame; node != NULL; node = node->next)
+	{
+		fr = node->data;
+		size += fr->fr_size + ID3_FRAMEHDR_SIZE;
+	}
+	
+	/*
+	 * Write tag header.
+	 */
+	buf[0] = id3->id3_version;
+	buf[1] = id3->id3_revision;
+	buf[2] = id3->id3_flags;
+	ID3_SET_SIZE28(size, buf[3], buf[4], buf[5], buf[6]);
+	
+	if (safe_write(fd, "ID3", 3) == -1)
+		return -1;
+	if (safe_write(fd, buf, ID3_TAGHDR_SIZE) == -1)
+		return -1;
 
-	if ( safe_write(fd, &fhdr, sizeof(fhdr)) == -1 )
-	    return -1;
-
-	if ( safe_write(fd, fr->fr_data, fr->fr_size) == -1 )
-	    return -1;
-    }
-    return 0;
+	/*
+	 * TODO: Write extended header.
+	 */
+#if 0
+	if (id3->id3_flags & ID3_THFLAG_EXT)
+	{
+		id3_exthdr_t exthdr;
+	}
+#endif
+	    
+	for (node = id3->id3_frame; node != NULL; node = node->next)
+	{
+		char fhdr[ID3_FRAMEHDR_SIZE];
+		
+		fr = node->data;
+		
+		/*
+		 * TODO: Support compressed headers, encoded
+		 * headers, and grouping info.
+		 */
+		/*  fhdr.fh_id = fr->fr_desc ? g_htonl(fr->fr_desc->fd_id) : 0; */
+		fhdr[3] = (fr->fr_size >> 24) & 0xff;
+		fhdr[4] = (fr->fr_size >> 16) & 0xff;
+		fhdr[5] = (fr->fr_size >> 8) & 0xff;
+		fhdr[6] = fr->fr_size & 0xff;
+		fhdr[7] = (fr->fr_flags >> 8) & 0xff;
+		fhdr[8] = fr->fr_flags & 0xff;
+		
+		if (safe_write(fd, fhdr, sizeof(fhdr)) == -1)
+			return -1;
+		
+		if (safe_write(fd, fr->fr_data, fr->fr_size) == -1)
+			return -1;
+	}
+	return 0;
 }
