@@ -17,6 +17,7 @@
  */
 #include "wav.h"
 #include "libxmms/util.h"
+#include "libxmms/titlestring.h"
 #include "xmms/i18n.h"
 
 InputPlugin wav_ip =
@@ -47,7 +48,7 @@ InputPlugin wav_ip =
 	NULL
 };
 
-WaveFile *wav_file = NULL;
+static WaveFile *wav_file = NULL;
 static pthread_t decode_thread;
 static gboolean audio_error = FALSE;
 
@@ -181,6 +182,37 @@ static int is_our_file(char *filename)
 	return FALSE;
 }
 
+static gchar *get_title(gchar *filename)
+{
+	TitleInput *input;
+	gchar *temp, *ext, *title, *path, *temp2;
+
+	XMMS_NEW_TITLEINPUT(input);
+
+	path = g_strdup(filename);
+	temp = g_strdup(filename);
+	ext = strrchr(temp, '.');
+	if (ext)
+		*ext = '\0';
+	temp2 = strrchr(path, '/');
+	if (temp2)
+		*temp2 = '\0';
+
+	input->file_name = g_basename(filename);
+	input->file_ext = ext ? ext+1 : NULL;
+	input->file_path = g_strdup_printf("%s/", path);
+
+	title = xmms_get_titlestring(xmms_get_gentitle_format(), input);
+	if ( title == NULL )
+		title = g_strdup(input->file_name);
+
+	g_free(temp);
+	g_free(path);
+	g_free(input);
+
+	return title;
+}
+
 static int read_le_long(FILE * file, long *ret)
 {
 	unsigned char buf[4];
@@ -205,12 +237,14 @@ static int read_le_short(FILE * file, short *ret)
 
 static void *play_loop(void *arg)
 {
-	char data[2048 * 2];
+	char *data;
 	int bytes, blk_size, rate;
 	int actual_read;
 
 	blk_size = 512 * (wav_file->bits_per_sample / 8) * wav_file->channels;
-	rate = wav_file->samples_per_sec * wav_file->channels * (wav_file->bits_per_sample / 8);
+	rate = wav_file->samples_per_sec * wav_file->channels *
+		(wav_file->bits_per_sample / 8);
+	data = g_malloc(blk_size);
 	while (wav_file->going)
 	{
 		if (!wav_file->eof)
@@ -222,7 +256,7 @@ static void *play_loop(void *arg)
 			{
 				actual_read = fread(data, 1, bytes, wav_file->file);
 
-				if (actual_read == -1)
+				if (actual_read == 0)
 				{
 					wav_file->eof = 1;
 					wav_ip.output->buffer_free();
@@ -258,27 +292,27 @@ static void *play_loop(void *arg)
 		}
 
 	}
+	g_free(data);
 	fclose(wav_file->file);
 	pthread_exit(NULL);
 }
 
 static void play_file(char *filename)
 {
-	char magic[4], *name, *temp;
+	char magic[4], *name;
 	unsigned long len;
 	int rate;
 
 	audio_error = FALSE;
 
-	wav_file = malloc(sizeof (WaveFile));
-	memset(wav_file, 0, sizeof (WaveFile));
+	wav_file = g_malloc0(sizeof (WaveFile));
 	if ((wav_file->file = fopen(filename, "rb")))
 	{
 		fread(magic, 1, 4, wav_file->file);
 		if (strncmp(magic, "RIFF", 4))
 		{
 			fclose(wav_file->file);
-			free(wav_file);
+			g_free(wav_file);
 			wav_file = NULL;
 			return;
 		}
@@ -287,7 +321,7 @@ static void play_file(char *filename)
 		if (strncmp(magic, "WAVE", 4))
 		{
 			fclose(wav_file->file);
-			free(wav_file);
+			g_free(wav_file);
 			wav_file = NULL;
 			return;
 		}
@@ -297,7 +331,7 @@ static void play_file(char *filename)
 			if (!read_le_long(wav_file->file, &len))
 			{
 				fclose(wav_file->file);
-				free(wav_file);
+				g_free(wav_file);
 				wav_file = NULL;
 				return;
 			}
@@ -308,7 +342,7 @@ static void play_file(char *filename)
 		if (len < 16)
 		{
 			fclose(wav_file->file);
-			free(wav_file);
+			g_free(wav_file);
 			wav_file = NULL;
 			return;
 		}
@@ -326,7 +360,7 @@ static void play_file(char *filename)
 			case IBM_FORMAT_ALAW:
 			case IBM_FORMAT_ADPCM:
 				fclose(wav_file->file);
-				free(wav_file);
+				g_free(wav_file);
 				wav_file = NULL;
 				return;
 		}
@@ -338,7 +372,7 @@ static void play_file(char *filename)
 		if (wav_file->bits_per_sample != 8 && wav_file->bits_per_sample != 16)
 		{
 			fclose(wav_file->file);
-			free(wav_file);
+			g_free(wav_file);
 			wav_file = NULL;
 			return;
 		}
@@ -353,7 +387,7 @@ static void play_file(char *filename)
 			if (!read_le_long(wav_file->file, &len))
 			{
 				fclose(wav_file->file);
-				free(wav_file);
+				g_free(wav_file);
 				wav_file = NULL;
 				return;
 			}
@@ -371,40 +405,28 @@ static void play_file(char *filename)
 		{
 			audio_error = TRUE;
 			fclose(wav_file->file);
-			free(wav_file);
+			g_free(wav_file);
 			wav_file = NULL;
 			return;
 		}
-		temp = strrchr(filename, '/');
-		if (!temp)
-			temp = filename;
-		else
-			temp++;
-		name = malloc(strlen(temp) + 1);
-		strcpy(name, temp);
-		*strrchr(name, '.') = '\0';
+		name = get_title(filename);
 		rate = wav_file->samples_per_sec * wav_file->channels * (wav_file->bits_per_sample / 8);
 		wav_ip.set_info(name, 1000 * (wav_file->length / rate), 8 * rate, wav_file->samples_per_sec, wav_file->channels);
-		free(name);
+		g_free(name);
 		wav_file->seek_to = -1;
 		pthread_create(&decode_thread, NULL, play_loop, NULL);
-
 	}
 }
 
 static void stop(void)
 {
-	if (wav_file)
+	if (wav_file && wav_file->going)
 	{
-		if (wav_file->going)
-		{
-			wav_file->going = 0;
-			pthread_join(decode_thread, NULL);
-			wav_ip.output->close_audio();
-			free(wav_file);
-			wav_file = NULL;
-
-		}
+		wav_file->going = 0;
+		pthread_join(decode_thread, NULL);
+		wav_ip.output->close_audio();
+		g_free(wav_file);
+		wav_file = NULL;
 	}
 }
 
@@ -439,12 +461,12 @@ static int get_time(void)
 
 static void get_song_info(char *filename, char **title, int *length)
 {
-	char magic[4], *name, *temp;
+	char magic[4];
 	unsigned long len;
 	int rate;
 	WaveFile *wav_file;
 
-	wav_file = malloc(sizeof (WaveFile));
+	wav_file = g_malloc(sizeof (WaveFile));
 	memset(wav_file, 0, sizeof (WaveFile));
 	if (!(wav_file->file = fopen(filename, "rb")))
 		return;
@@ -453,7 +475,7 @@ static void get_song_info(char *filename, char **title, int *length)
 	if (strncmp(magic, "RIFF", 4))
 	{
 		fclose(wav_file->file);
-		free(wav_file);
+		g_free(wav_file);
 		wav_file = NULL;
 		return;
 	}
@@ -462,7 +484,7 @@ static void get_song_info(char *filename, char **title, int *length)
 	if (strncmp(magic, "WAVE", 4))
 	{
 		fclose(wav_file->file);
-		free(wav_file);
+		g_free(wav_file);
 		wav_file = NULL;
 		return;
 	}
@@ -472,7 +494,7 @@ static void get_song_info(char *filename, char **title, int *length)
 		if (!read_le_long(wav_file->file, &len))
 		{
 			fclose(wav_file->file);
-			free(wav_file);
+			g_free(wav_file);
 			wav_file = NULL;
 			return;
 		}
@@ -483,7 +505,7 @@ static void get_song_info(char *filename, char **title, int *length)
 	if (len < 16)
 	{
 		fclose(wav_file->file);
-		free(wav_file);
+		g_free(wav_file);
 		wav_file = NULL;
 		return;
 	}
@@ -501,7 +523,7 @@ static void get_song_info(char *filename, char **title, int *length)
 		case IBM_FORMAT_ALAW:
 		case IBM_FORMAT_ADPCM:
 			fclose(wav_file->file);
-			free(wav_file);
+			g_free(wav_file);
 			wav_file = NULL;
 			return;
 	}
@@ -513,7 +535,7 @@ static void get_song_info(char *filename, char **title, int *length)
 	if (wav_file->bits_per_sample != 8 && wav_file->bits_per_sample != 16)
 	{
 		fclose(wav_file->file);
-		free(wav_file);
+		g_free(wav_file);
 		wav_file = NULL;
 		return;
 	}
@@ -528,7 +550,7 @@ static void get_song_info(char *filename, char **title, int *length)
 		if (!read_le_long(wav_file->file, &len))
 		{
 			fclose(wav_file->file);
-			free(wav_file);
+			g_free(wav_file);
 			wav_file = NULL;
 			return;
 		}
@@ -538,19 +560,9 @@ static void get_song_info(char *filename, char **title, int *length)
 	}
 	rate = wav_file->samples_per_sec * wav_file->channels * (wav_file->bits_per_sample / 8);
 	(*length) = 1000 * (len / rate);
-
-	temp = strrchr(filename, '/');
-	if (!temp)
-		temp = filename;
-	else
-		temp++;
-	name = g_malloc(strlen(temp) + 1);
-	strcpy(name, temp);
-	*strrchr(name, '.') = '\0';
-	(*title) = name;
+	(*title) = get_title(filename);
 
 	fclose(wav_file->file);
-	free(wav_file);
+	g_free(wav_file);
 	wav_file = NULL;
-
 }
