@@ -1,3 +1,5 @@
+#include "config.h"
+
 #include <gtk/gtk.h>
 
 #include "libxmms/configfile.h"
@@ -35,7 +37,7 @@ static void configure_ok(GtkWidget *w, gpointer data)
 
 static void configure_cancel(GtkWidget *w, gpointer data)
 {
-	bscope_cfg.color = (guint32)data;
+	bscope_cfg.color = GPOINTER_TO_UINT(data);
 	generate_cmap();
 	gtk_widget_destroy(configure_win);
 }
@@ -107,7 +109,7 @@ void bscope_configure (void)
 	cancel = gtk_button_new_with_label(_("Cancel"));
 	gtk_signal_connect(GTK_OBJECT(cancel), "clicked",
 			   GTK_SIGNAL_FUNC(configure_cancel),
-			   (gpointer)bscope_cfg.color);
+			   GUINT_TO_POINTER(bscope_cfg.color));
 	GTK_WIDGET_SET_FLAGS(cancel, GTK_CAN_DEFAULT);
 	gtk_box_pack_start(GTK_BOX(bbox), cancel, TRUE, TRUE, 0);
 	gtk_widget_show(cancel);
