@@ -1,5 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
- *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
+ *  Copyright (C) 1998-2002  Peter Alm, Mikael Alm, Olle Hallnas,
+ *                           Thomas Nilsson and 4Front Technologies
+ *  Copyright (C) 1999-2004  Haavard Kvaalen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -35,7 +37,7 @@ GdkGC *equalizerwin_gc;
 
 GList *equalizerwin_wlist = NULL;
 
-GtkAccelGroup *equalizerwin_accel;
+static GtkAccelGroup *equalizerwin_accel;
 
 static TButton *equalizerwin_on, *equalizerwin_auto;
 extern TButton *mainwin_eq;
@@ -58,8 +60,6 @@ EqualizerPreset;
 
 static GList *equalizer_presets = NULL, *equalizer_auto_presets = NULL;
 
-#define EQUALIZERWIN_PRESETS_MENU_ENTRIES 20
-
 void equalizerwin_presets_menu_cb(gpointer cb_data, guint action, GtkWidget * w);
 GtkWidget * equalizerwin_create_conf_window(void);
 
@@ -68,6 +68,7 @@ enum
 	EQUALIZER_PRESETS_LOAD_PRESET, EQUALIZER_PRESETS_LOAD_AUTOPRESET,
 	EQUALIZER_PRESETS_LOAD_DEFAULT,	EQUALIZER_PRESETS_LOAD_ZERO,
 	EQUALIZER_PRESETS_LOAD_FROM_FILE, EQUALIZER_PRESETS_LOAD_FROM_WINAMPFILE,
+	EQUALIZER_PRESETS_IMPORT_WINAMPFILE,
 	EQUALIZER_PRESETS_SAVE_PRESET, EQUALIZER_PRESETS_SAVE_AUTOPRESET,
 	EQUALIZER_PRESETS_SAVE_DEFAULT,	EQUALIZER_PRESETS_SAVE_TO_FILE,
 	EQUALIZER_PRESETS_SAVE_TO_WINAMPFILE, EQUALIZER_PRESETS_DELETE_PRESET,
@@ -85,6 +86,8 @@ GtkItemFactoryEntry equalizerwin_presets_menu_entries[] =
 	{N_("/Load/-"), NULL, NULL, 0, "<Separator>"},
 	{N_("/Load/From file"), NULL, equalizerwin_presets_menu_cb, EQUALIZER_PRESETS_LOAD_FROM_FILE, "<Item>"},
 	{N_("/Load/From WinAMP EQF file"), NULL, equalizerwin_presets_menu_cb, EQUALIZER_PRESETS_LOAD_FROM_WINAMPFILE, "<Item>"},
+	{N_("/Import"), NULL, NULL, 0, "<Branch>"},
+	{N_("/Import/WinAMP Presets"), NULL, equalizerwin_presets_menu_cb, EQUALIZER_PRESETS_IMPORT_WINAMPFILE, "<Item>"},
 	{N_("/Save"), NULL, NULL, 0, "<Branch>"},
 	{N_("/Save/Preset"), NULL, equalizerwin_presets_menu_cb, EQUALIZER_PRESETS_SAVE_PRESET, "<Item>"},
 	{N_("/Save/Auto-load preset"), NULL, equalizerwin_presets_menu_cb, EQUALIZER_PRESETS_SAVE_AUTOPRESET, "<Item>"},
@@ -98,6 +101,18 @@ GtkItemFactoryEntry equalizerwin_presets_menu_entries[] =
 	{N_("/Configure Equalizer"), NULL, equalizerwin_presets_menu_cb, EQUALIZER_PRESETS_CONFIGURE, "<Item>"},
 };
 
+static gint equalizerwin_presets_menu_entries_num = 
+	sizeof(equalizerwin_presets_menu_entries) / 
+	sizeof(equalizerwin_presets_menu_entries[0]);
+
+void equalizerwin_set_shape_mask(void)
+{
+	if (cfg.show_wm_decorations)
+		return;
+
+	gtk_widget_shape_combine_mask(equalizerwin, skin_get_mask(SKIN_MASK_EQ, EQUALIZER_DOUBLESIZE, cfg.equalizer_shaded), 0, 0);
+}
+
 void equalizerwin_set_doublesize(gboolean ds)
 {
 	gint height;
@@ -107,7 +122,7 @@ void equalizerwin_set_doublesize(gboolean ds)
 	else
 		height = 116;
 
-	gtk_widget_shape_combine_mask(equalizerwin, skin_get_mask(SKIN_MASK_EQ, ds, cfg.equalizer_shaded), 0, 0);
+	equalizerwin_set_shape_mask();
 
 	if (ds)
 	{
@@ -122,9 +137,11 @@ void equalizerwin_set_doublesize(gboolean ds)
 	draw_equalizer_window(TRUE);
 }
 
-void equalizerwin_set_shade(gboolean shaded)
+void equalizerwin_set_shade_menu_cb(gboolean shaded)
 {
-	gtk_widget_shape_combine_mask(equalizerwin, skin_get_mask(SKIN_MASK_EQ, EQUALIZER_DOUBLESIZE, shaded), 0, 0);
+	cfg.equalizer_shaded = shaded;
+
+	equalizerwin_set_shape_mask();
 	
 	if (shaded)
 	{
@@ -147,13 +164,20 @@ void equalizerwin_set_shade(gboolean shaded)
 		hide_widget(equalizerwin_balance);
 	}
 	
-	cfg.equalizer_shaded = shaded;
 	draw_equalizer_window(TRUE);
 }
 
-void equalizerwin_shade_cb(void)
+void equalizerwin_shade_toggle(void)
 {
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu, _("/Equalizer WindowShade Mode"))), !cfg.equalizer_shaded);
+	equalizerwin_set_shade(!cfg.equalizer_shaded);
+}
+
+void equalizerwin_set_shade(gboolean shaded)
+{
+	GtkWidget *widget;
+        widget = gtk_item_factory_get_widget(mainwin_options_menu,
+					     "/Equalizer WindowShade Mode");
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(widget), shaded);
 }
 
 void equalizerwin_raise(void)
@@ -162,15 +186,6 @@ void equalizerwin_raise(void)
 		gdk_window_raise(equalizerwin->window);
 }
 
-void equalizerwin_move(gint x, gint y)
-{
-	cfg.equalizer_x = x;
-	cfg.equalizer_y = y;
-	if (cfg.equalizer_visible)
-		gdk_window_move(equalizerwin->window, x, y);
-		
-}
-
 void equalizerwin_eq_changed(void)
 {
 	int i;
@@ -316,13 +331,21 @@ void equalizerwin_press(GtkWidget * widget, GdkEventButton * event, gpointer cal
 	    ((cfg.easy_move || cfg.equalizer_shaded || event->y < 14) &&
 	     !inside_sensitive_widgets(event->x, event->y)))
 	{
-
-		equalizerwin_raise();
-		dock_move_press(dock_window_list, equalizerwin, event, FALSE);
+		if (0 && hint_move_resize_available())
+		{
+			hint_move_resize(equalizerwin, event->x_root,
+					 event->y_root, TRUE);
+			grab = FALSE;
+		}
+		else
+		{
+			equalizerwin_raise();
+			dock_move_press(dock_window_list, equalizerwin, event, FALSE);
+		}
 	}
 	else if (event->button == 1 && event->type == GDK_2BUTTON_PRESS && event->y < 14)
 	{
-		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu, _("/Equalizer WindowShade Mode"))), !cfg.equalizer_shaded);
+		equalizerwin_set_shade(!cfg.equalizer_shaded);
 		if(dock_is_moving(equalizerwin))
 			dock_move_release(equalizerwin);
 	}
@@ -334,7 +357,7 @@ void equalizerwin_press(GtkWidget * widget, GdkEventButton * event, gpointer cal
 		 * Pop up the main menu a few pixels down to avoid
 		 * anything to be selected initially.
 		 */
-		util_item_factory_popup(mainwin_general_menu, event->x_root, event->y_root + 2, 3, GDK_CURRENT_TIME);
+		util_item_factory_popup(mainwin_general_menu, event->x_root, event->y_root + 2, 3, event->time);
 		grab = FALSE;
 	}
 	else
@@ -402,26 +425,107 @@ void equalizerwin_focus_out(GtkWidget * widget, GdkEventButton * event, gpointer
 	draw_equalizer_window(TRUE);
 }
 
+#define EQ_SLIDER_DOWN(slider)							\
+do {										\
+	eqslider_set_position(slider, eqslider_get_position(slider) - 1.6f);	\
+	equalizerwin_eq_changed();						\
+} while (0)
+	
+#define EQ_SLIDER_UP(slider)							\
+do {										\
+	eqslider_set_position(slider, eqslider_get_position(slider) + 1.6f);	\
+	equalizerwin_eq_changed();						\
+} while (0)
+
 gboolean equalizerwin_keypress(GtkWidget * w, GdkEventKey * event, gpointer data)
 {
-	if(!cfg.equalizer_shaded)
-	{
-		gtk_widget_event(mainwin, (GdkEvent *) event);
-		return TRUE;
-	}
-	
-	switch(event->keyval)
+	switch (event->keyval)
 	{
 		case GDK_Left:
 		case GDK_KP_Left:
-			mainwin_set_balance_diff(-4);
+			if (cfg.equalizer_shaded)
+				mainwin_set_balance_diff(-4);
+			else
+				gtk_widget_event(mainwin, (GdkEvent*) event);
 			break;
 		case GDK_Right:
 		case GDK_KP_Right:
-			mainwin_set_balance_diff(4);
+			if (cfg.equalizer_shaded)
+				mainwin_set_balance_diff(4);
+			else
+				gtk_widget_event(mainwin, (GdkEvent*) event);
+			break;
+		case GDK_quoteleft:
+			EQ_SLIDER_UP(equalizerwin_preamp);
+			break;
+		case GDK_Tab:
+			EQ_SLIDER_DOWN(equalizerwin_preamp);
+			break;
+		case GDK_1:
+			EQ_SLIDER_UP(equalizerwin_bands[0]);
+			break;
+		case GDK_q:
+			EQ_SLIDER_DOWN(equalizerwin_bands[0]);
+			break;
+		case GDK_2:
+			EQ_SLIDER_UP(equalizerwin_bands[1]);
+			break;
+		case GDK_w:
+			EQ_SLIDER_DOWN(equalizerwin_bands[1]);
+			break;
+		case GDK_3:
+			EQ_SLIDER_UP(equalizerwin_bands[2]);
+			break;
+		case GDK_e:
+			EQ_SLIDER_DOWN(equalizerwin_bands[2]);
+			break;
+		case GDK_4:
+			EQ_SLIDER_UP(equalizerwin_bands[3]);
+			break;
+		case GDK_r:
+			EQ_SLIDER_DOWN(equalizerwin_bands[3]);
+			break;
+		case GDK_5:
+			EQ_SLIDER_UP(equalizerwin_bands[4]);
+			break;
+		case GDK_t:
+			EQ_SLIDER_DOWN(equalizerwin_bands[4]);
+			break;
+		case GDK_6:
+			EQ_SLIDER_UP(equalizerwin_bands[5]);
+			break;
+		case GDK_y:
+			EQ_SLIDER_DOWN(equalizerwin_bands[5]);
+			break;
+		case GDK_7:
+			EQ_SLIDER_UP(equalizerwin_bands[6]);
+			break;
+		case GDK_u:
+			EQ_SLIDER_DOWN(equalizerwin_bands[6]);
+			break;
+		case GDK_8:
+			EQ_SLIDER_UP(equalizerwin_bands[7]);
+			break;
+		case GDK_i:
+			EQ_SLIDER_DOWN(equalizerwin_bands[7]);
+			break;
+		case GDK_9:
+			EQ_SLIDER_UP(equalizerwin_bands[8]);
+			break;
+		case GDK_o:
+			EQ_SLIDER_DOWN(equalizerwin_bands[8]);
+			break;
+		case GDK_0:
+			EQ_SLIDER_UP(equalizerwin_bands[9]);
+			break;
+		case GDK_p:
+			EQ_SLIDER_DOWN(equalizerwin_bands[9]);
 			break;
 		default:
-			gtk_widget_event(mainwin, (GdkEvent *) event);
+			if (!gtk_accel_group_activate(equalizerwin_accel,
+						      event->keyval,
+						      event->state))
+				gtk_widget_event(mainwin, (GdkEvent*) event);
 			break;
 	}
 
@@ -433,7 +537,13 @@ static gboolean equalizerwin_configure(GtkWidget * window, GdkEventConfigure *ev
 	if (!GTK_WIDGET_VISIBLE(window))
 		return FALSE;
 
-	gdk_window_get_deskrelative_origin(window->window, &cfg.equalizer_x, &cfg.equalizer_y);
+	if (cfg.show_wm_decorations)
+		gdk_window_get_root_origin(window->window,
+					   &cfg.equalizer_x, &cfg.equalizer_y);
+	else
+		gdk_window_get_deskrelative_origin(window->window,
+						   &cfg.equalizer_x,
+						   &cfg.equalizer_y);
 	return FALSE;
 }
 
@@ -584,10 +694,52 @@ void equalizerwin_set_volume_slider(gint percent)
 	hslider_set_position(equalizerwin_volume, (gint) rint(percent*94/100.0));
 }
 
-void equalizerwin_create(void)
+static void equalizerwin_create_widgets(void)
 {
 	int i;
 
+	equalizerwin_on = create_tbutton(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 14, 18, 25, 12, 10, 119, 128, 119, 69, 119, 187, 119, equalizerwin_on_pushed, SKIN_EQMAIN);
+	tbutton_set_toggled(equalizerwin_on, cfg.equalizer_active);
+	equalizerwin_auto = create_tbutton(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 39, 18, 33, 12, 35, 119, 153, 119, 94, 119, 212, 119, equalizerwin_auto_pushed, SKIN_EQMAIN);
+	tbutton_set_toggled(equalizerwin_auto, cfg.equalizer_autoload);
+	equalizerwin_presets = create_pbutton(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 217, 18, 44, 12, 224, 164, 224, 176, equalizerwin_presets_pushed, SKIN_EQMAIN);
+	equalizerwin_close = create_pbutton(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 264, 3, 9, 9, 0, 116, 0, 125, equalizerwin_close_cb, SKIN_EQMAIN);
+	equalizerwin_close->pb_allow_draw = FALSE;
+
+	equalizerwin_shade = create_pbutton_ex(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 254, 3, 9, 9, 254, 137, 1, 38, equalizerwin_shade_toggle, SKIN_EQMAIN, SKIN_EQ_EX);
+	equalizerwin_shade->pb_allow_draw = FALSE;
+
+	equalizerwin_graph = create_eqgraph(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 86, 17);
+	equalizerwin_preamp = create_eqslider(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 21, 38);
+	eqslider_set_position(equalizerwin_preamp, cfg.equalizer_preamp);
+	for (i = 0; i < 10; i++)
+	{
+		equalizerwin_bands[i] = create_eqslider(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 78 + (i * 18), 38);
+		eqslider_set_position(equalizerwin_bands[i], cfg.equalizer_bands[i]);
+	}
+
+	equalizerwin_volume = create_hslider(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 61, 4, 97, 8, 1, 30, 1, 30, 3, 7, 4, 61, 0, 94, equalizerwin_volume_frame_cb, equalizerwin_volume_motion_cb, equalizerwin_volume_release_cb, SKIN_EQ_EX);
+	equalizerwin_balance = create_hslider(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 164, 4, 42, 8, 11, 30, 11, 30, 3, 7, 4, 164, 0, 39, equalizerwin_balance_frame_cb, equalizerwin_balance_motion_cb, equalizerwin_balance_release_cb, SKIN_EQ_EX);
+
+	if (!cfg.equalizer_shaded)
+	{
+		hide_widget(equalizerwin_volume);
+		hide_widget(equalizerwin_balance);
+	}
+	else
+	{
+		pbutton_set_button_data(equalizerwin_shade, -1, 3, -1, 47);
+		pbutton_set_skin_index1(equalizerwin_shade, SKIN_EQ_EX);
+		pbutton_set_button_data(equalizerwin_close, 11, 38, 11, 47);
+		pbutton_set_skin_index(equalizerwin_close, SKIN_EQ_EX);
+	}
+
+
+}
+
+
+static void equalizerwin_create_gtk(void)
+{
 	equalizerwin = gtk_window_new(GTK_WINDOW_DIALOG);
 	dock_add_window(dock_window_list, equalizerwin);
 	gtk_widget_set_app_paintable(equalizerwin, TRUE);
@@ -605,14 +757,13 @@ void equalizerwin_create(void)
 	gtk_widget_set_events(equalizerwin, GDK_FOCUS_CHANGE_MASK | GDK_BUTTON_MOTION_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
 	gtk_widget_realize(equalizerwin);
 	hint_set_skip_winlist(equalizerwin);
-	gdk_window_set_decorations(equalizerwin->window, 0);
+	util_set_cursor(equalizerwin);
+	if (!cfg.show_wm_decorations)
+		gdk_window_set_decorations(equalizerwin->window, 0);
 
-	equalizerwin_accel = gtk_accel_group_new();
 	gtk_window_add_accel_group(GTK_WINDOW(equalizerwin), equalizerwin_accel);
 
-	equalizerwin_gc = gdk_gc_new(equalizerwin->window);
-	equalizerwin_bg = gdk_pixmap_new(equalizerwin->window, 275, 116, gdk_rgb_get_visual()->depth);
-	equalizerwin_bg_dblsize = gdk_pixmap_new(equalizerwin->window, 550, 232, gdk_rgb_get_visual()->depth);
+	equalizerwin_set_back_pixmap();
 	if (cfg.doublesize && cfg.eq_doublesize_linked)
 		gdk_window_set_back_pixmap(equalizerwin->window, equalizerwin_bg_dblsize, 0);
 	else
@@ -636,72 +787,50 @@ void equalizerwin_create(void)
 			   GTK_SIGNAL_FUNC(equalizerwin_client_event), NULL);
 	gtk_signal_connect(GTK_OBJECT(equalizerwin), "key-press-event",
 			   GTK_SIGNAL_FUNC(equalizerwin_keypress), NULL);
-
-	equalizerwin_on = create_tbutton(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 14, 18, 25, 12, 10, 119, 128, 119, 69, 119, 187, 119, equalizerwin_on_pushed, SKIN_EQMAIN);
-	tbutton_set_toggled(equalizerwin_on, cfg.equalizer_active);
-	equalizerwin_auto = create_tbutton(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 39, 18, 33, 12, 35, 119, 153, 119, 94, 119, 212, 119, equalizerwin_auto_pushed, SKIN_EQMAIN);
-	tbutton_set_toggled(equalizerwin_auto, cfg.equalizer_autoload);
-	equalizerwin_presets = create_pbutton(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 217, 18, 44, 12, 224, 164, 224, 176, equalizerwin_presets_pushed, SKIN_EQMAIN);
-	equalizerwin_close = create_pbutton(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 264, 3, 9, 9, 0, 116, 0, 125, equalizerwin_close_cb, SKIN_EQMAIN);
-	equalizerwin_close->pb_allow_draw = FALSE;
-
-	equalizerwin_shade = create_pbutton_ex(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 254, 3, 9, 9, 254, 137, 1, 38, equalizerwin_shade_cb, SKIN_EQMAIN, SKIN_EQ_EX);
-	equalizerwin_shade->pb_allow_draw = FALSE;
-
-	equalizerwin_graph = create_eqgraph(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 86, 17);
-	equalizerwin_preamp = create_eqslider(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 21, 38);
-	eqslider_set_position(equalizerwin_preamp, cfg.equalizer_preamp);
-	for (i = 0; i < 10; i++)
-	{
-		equalizerwin_bands[i] = create_eqslider(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 78 + (i * 18), 38);
-		eqslider_set_position(equalizerwin_bands[i], cfg.equalizer_bands[i]);
-	}
-
-	equalizerwin_volume = create_hslider(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 61, 4, 97, 8, 1, 30, 1, 30, 3, 7, 4, 61, 0, 94, equalizerwin_volume_frame_cb, equalizerwin_volume_motion_cb, equalizerwin_volume_release_cb, SKIN_EQ_EX);
-	equalizerwin_balance = create_hslider(&equalizerwin_wlist, equalizerwin_bg, equalizerwin_gc, 164, 4, 42, 8, 11, 30, 11, 30, 3, 7, 4, 164, 0, 39, equalizerwin_balance_frame_cb, equalizerwin_balance_motion_cb, equalizerwin_balance_release_cb, SKIN_EQ_EX);
-
-	equalizerwin_presets_menu = gtk_item_factory_new(GTK_TYPE_MENU, "<Main>", equalizerwin_accel);
-#ifdef ENABLE_NLS
-{
-	int i;
-	for (i=0;i<EQUALIZERWIN_PRESETS_MENU_ENTRIES;i++) {
-		equalizerwin_presets_menu_entries[i].path =
-			gettext(equalizerwin_presets_menu_entries[i].path);
-	}
 }
-#endif
-	gtk_item_factory_create_items(equalizerwin_presets_menu, EQUALIZERWIN_PRESETS_MENU_ENTRIES, equalizerwin_presets_menu_entries, NULL);
+
+void equalizerwin_create(void)
+{
+	equalizerwin_accel = gtk_accel_group_new();
+	equalizerwin_presets_menu = gtk_item_factory_new(GTK_TYPE_MENU,
+							 "<Main>",
+							 equalizerwin_accel);
+	gtk_item_factory_set_translate_func(equalizerwin_presets_menu,
+					    util_menu_translate, NULL, NULL);
+	gtk_item_factory_create_items(equalizerwin_presets_menu, equalizerwin_presets_menu_entries_num, equalizerwin_presets_menu_entries, NULL);
 	equalizer_presets = equalizerwin_read_presets("eq.preset");
 	equalizer_auto_presets = equalizerwin_read_presets("eq.auto_preset");
-	if (!cfg.equalizer_shaded)
-	{
-		hide_widget(equalizerwin_volume);
-		hide_widget(equalizerwin_balance);
-	}
-	else
-	{
-		pbutton_set_button_data(equalizerwin_shade, -1, 3, -1, 47);
-		pbutton_set_skin_index1(equalizerwin_shade, SKIN_EQ_EX);
-		pbutton_set_button_data(equalizerwin_close, 11, 38, 11, 47);
-		pbutton_set_skin_index(equalizerwin_close, SKIN_EQ_EX);
-	}
 
+	equalizerwin_bg = gdk_pixmap_new(NULL, 275, 116, gdk_rgb_get_visual()->depth);
+	equalizerwin_bg_dblsize = gdk_pixmap_new(NULL, 550, 232, gdk_rgb_get_visual()->depth);
+	equalizerwin_create_gtk();
+	equalizerwin_gc = gdk_gc_new(equalizerwin->window);
+	equalizerwin_create_widgets();
+}
+
+void equalizerwin_recreate(void)
+{
+	dock_window_list = g_list_remove(dock_window_list, equalizerwin);
+	gtk_widget_destroy(equalizerwin);
+	equalizerwin_create_gtk();
+
+	equalizerwin_set_shape_mask();
 }
 
 void equalizerwin_show(gboolean show)
 {
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_general_menu, _("/Graphical EQ"))), show);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_general_menu, "/Graphical EQ")), show);
 }
 
-
-
-
 void equalizerwin_real_show(void)
 {
 	/*
 	 * This function should only be called from the
 	 * main menu signal handler
 	 */
+	if (!pposition_broken && cfg.equalizer_x != -1 &&
+	    cfg.save_window_position && cfg.show_wm_decorations)
+		dock_set_uposition(equalizerwin, cfg.equalizer_x, cfg.equalizer_y);
 	gtk_widget_show(equalizerwin);
 	if (pposition_broken && cfg.equalizer_x != -1 && cfg.save_window_position)
 		dock_set_uposition(equalizerwin, cfg.equalizer_x, cfg.equalizer_y);
@@ -713,7 +842,7 @@ void equalizerwin_real_show(void)
 	draw_equalizer_window(TRUE);
 	cfg.equalizer_visible = TRUE;
 	tbutton_set_toggled(mainwin_eq, TRUE);
-	mainwin_set_always_on_top(cfg.always_on_top);
+	hint_set_always(cfg.always_on_top);
 	hint_set_sticky(cfg.sticky);
 	hint_set_skip_winlist(equalizerwin);
 }
@@ -831,6 +960,39 @@ static GList *equalizerwin_delete_preset(GList * list, gchar * name, gchar * fna
 	return list;
 }
 
+
+static GList *equalizerwin_import_winamp_eqf(FILE *file)
+{
+	gchar header[31];
+	gchar tmp[257];
+	gchar bands[11];
+	gint i=0;
+	GList *list = NULL;
+	EqualizerPreset *preset;
+
+	fread(header, 1, 31, file);
+	if (!strncmp(header, "Winamp EQ library file v1.1", 27))
+	{
+		while (fread(tmp, 1, 257, file)) {
+			preset = g_malloc(sizeof (EqualizerPreset));
+
+			fread(bands, 1, 11, file);
+
+			preset->name = g_strdup(tmp);
+			preset->preamp =  20.0 - ((bands[10] * 40.0) / 64);
+
+			for (i = 0; i < 10; i++)
+				preset->bands[i] = 20.0 - ((bands[i] * 40.0) / 64);
+
+			list = g_list_prepend(list, preset);
+		}
+        }
+
+	list = g_list_reverse(list);
+	return list;
+
+}
+
 static void equalizerwin_read_winamp_eqf(FILE *file)
 {
 	gchar header[31];
@@ -885,7 +1047,7 @@ static void equalizerwin_save_select(GtkCList * clist, gint row, gint column, Gd
 	gtk_clist_get_text(clist, row, 0, &text);
 
 	gtk_entry_set_text(GTK_ENTRY(equalizerwin_save_entry), text);
-	if (event->type == GDK_2BUTTON_PRESS)
+	if (event && event->type == GDK_2BUTTON_PRESS)
 		equalizerwin_save_ok(NULL, NULL);
 
 }
@@ -905,7 +1067,7 @@ static void equalizerwin_load_ok(GtkWidget * widget, gpointer data)
 
 static void equalizerwin_load_select(GtkCList * widget, gint row, gint column, GdkEventButton * event, gpointer data)
 {
-	if (event->type == GDK_2BUTTON_PRESS)
+	if (event && event->type == GDK_2BUTTON_PRESS)
 		equalizerwin_load_ok(NULL, widget);
 }
 
@@ -923,8 +1085,8 @@ static void equalizerwin_delete_delete(GtkWidget * widget, gpointer data)
 	{
 		next = g_list_next(list);
 		gtk_clist_get_text(clist, GPOINTER_TO_INT(list->data), 0, &text);
-		equalizer_auto_presets = equalizerwin_delete_preset(equalizer_presets,
-								    text, "eq.preset");
+		equalizer_presets = equalizerwin_delete_preset(equalizer_presets,
+							       text, "eq.preset");
 		gtk_clist_remove(clist, GPOINTER_TO_INT(list->data));
 		list = next;
 	}
@@ -948,7 +1110,7 @@ static void equalizerwin_save_auto_select(GtkCList * clist, gint row, gint colum
 	gtk_clist_get_text(clist, row, 0, &text);
 
 	gtk_entry_set_text(GTK_ENTRY(equalizerwin_save_auto_entry), text);
-	if (event->type == GDK_2BUTTON_PRESS)
+	if (event && event->type == GDK_2BUTTON_PRESS)
 		equalizerwin_save_auto_ok(NULL, NULL);
 
 }
@@ -968,7 +1130,7 @@ static void equalizerwin_load_auto_ok(GtkWidget * widget, gpointer data)
 
 static void equalizerwin_load_auto_select(GtkWidget * widget, gint row, gint column, GdkEventButton * event, gpointer data)
 {
-	if (event->type == GDK_2BUTTON_PRESS)
+	if (event && event->type == GDK_2BUTTON_PRESS)
 		equalizerwin_load_auto_ok(NULL, widget);
 }
 
@@ -1011,6 +1173,26 @@ static void equalizerwin_load_filesel_ok(GtkWidget * w, GtkFileSelection * files
 	gtk_widget_destroy(GTK_WIDGET(filesel));
 }
 
+static void equalizerwin_import_winamp_filesel_ok(GtkWidget * w, GtkFileSelection * filesel)
+{
+	gchar *filename;
+	FILE *file;
+
+	if (util_filebrowser_is_dir(filesel))
+		return;
+
+	filename = gtk_file_selection_get_filename(filesel);
+
+	if ((file = fopen(filename, "rb")) != NULL)
+	{
+		equalizer_presets = g_list_concat(equalizer_presets,
+				    equalizerwin_import_winamp_eqf(file));
+		equalizerwin_write_preset_file(equalizer_presets, "eq.preset");
+	}
+
+	gtk_widget_destroy(GTK_WIDGET(filesel));
+}
+
 static void equalizerwin_load_winamp_filesel_ok(GtkWidget * w, GtkFileSelection * filesel)
 {
 	gchar *filename;
@@ -1099,7 +1281,7 @@ static GtkWidget *equalizerwin_create_list_window(GList * preset_list,
 						  GtkSignalFunc select_row_func)
 {
 	GtkWidget *vbox, *scrolled_window, *bbox, *btn1, *btn2, *clist;
-	gchar *preset_text[] = {N_("Presets")};
+	char *preset_text[1];
 	GList *node;
 
 	*window = gtk_window_new(GTK_WINDOW_DIALOG);
@@ -1119,9 +1301,7 @@ static GtkWidget *equalizerwin_create_list_window(GList * preset_list,
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
 				       GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
 
-#ifdef ENABLE_NLS
-	preset_text[0] = gettext(preset_text[0]);
-#endif
+	preset_text[0] = _("Presets");
 	clist = gtk_clist_new_with_titles(1, preset_text);
 	if (select_row_func)
 		gtk_signal_connect(GTK_OBJECT(clist), "select_row",
@@ -1188,13 +1368,13 @@ void equalizerwin_presets_menu_cb(gpointer cb_data, guint action, GtkWidget * w)
 	{
 		case EQUALIZER_PRESETS_LOAD_PRESET:
 			if (!equalizerwin_load_window)
-				equalizerwin_create_list_window(equalizer_presets, _("Load preset"), &equalizerwin_load_window, GTK_SELECTION_SINGLE, NULL, _("Ok"), _("Cancel"), equalizerwin_load_ok, equalizerwin_load_select);
+				equalizerwin_create_list_window(equalizer_presets, _("Load preset"), &equalizerwin_load_window, GTK_SELECTION_SINGLE, NULL, _("OK"), _("Cancel"), equalizerwin_load_ok, equalizerwin_load_select);
 			else
 				gdk_window_raise(equalizerwin_load_window->window);
 			break;
 		case EQUALIZER_PRESETS_LOAD_AUTOPRESET:
 			if (!equalizerwin_load_auto_window)
-				equalizerwin_create_list_window(equalizer_auto_presets, _("Load auto-preset"), &equalizerwin_load_auto_window, GTK_SELECTION_SINGLE, NULL, _("Ok"), _("Cancel"), equalizerwin_load_auto_ok, equalizerwin_load_auto_select);
+				equalizerwin_create_list_window(equalizer_auto_presets, _("Load auto-preset"), &equalizerwin_load_auto_window, GTK_SELECTION_SINGLE, NULL, _("OK"), _("Cancel"), equalizerwin_load_auto_ok, equalizerwin_load_auto_select);
 			else
 				gdk_window_raise(equalizerwin_load_auto_window->window);
 			break;
@@ -1240,9 +1420,25 @@ void equalizerwin_presets_menu_cb(gpointer cb_data, guint action, GtkWidget * w)
 			
 			break;
 		}
+		case EQUALIZER_PRESETS_IMPORT_WINAMPFILE:
+		{
+			static GtkWidget *import_winamp_filesel;
+
+			if (import_winamp_filesel != NULL)
+				break;
+
+			import_winamp_filesel = gtk_file_selection_new(_("Import equalizer presets"));
+			gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(import_winamp_filesel)->ok_button), "clicked", GTK_SIGNAL_FUNC(equalizerwin_import_winamp_filesel_ok), import_winamp_filesel);
+			gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(import_winamp_filesel)->cancel_button), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(import_winamp_filesel));
+			gtk_signal_connect(GTK_OBJECT(import_winamp_filesel), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &import_winamp_filesel);
+			gtk_widget_show(import_winamp_filesel);
+			
+			break;
+		}
+	
 		case EQUALIZER_PRESETS_SAVE_PRESET:
 			if (!equalizerwin_save_window)
-				equalizerwin_create_list_window(equalizer_presets, _("Save preset"), &equalizerwin_save_window, GTK_SELECTION_SINGLE, &equalizerwin_save_entry, _("Ok"), _("Cancel"), equalizerwin_save_ok, equalizerwin_save_select);
+				equalizerwin_create_list_window(equalizer_presets, _("Save preset"), &equalizerwin_save_window, GTK_SELECTION_SINGLE, &equalizerwin_save_entry, _("OK"), _("Cancel"), equalizerwin_save_ok, equalizerwin_save_select);
 			else
 				gdk_window_raise(equalizerwin_save_window->window);
 			break;
@@ -1251,7 +1447,7 @@ void equalizerwin_presets_menu_cb(gpointer cb_data, guint action, GtkWidget * w)
 			gchar *name;
 
 			if (!equalizerwin_save_auto_window)
-				equalizerwin_create_list_window(equalizer_auto_presets, _("Save auto-preset"), &equalizerwin_save_auto_window, GTK_SELECTION_SINGLE, &equalizerwin_save_auto_entry, _("Ok"), _("Cancel"), equalizerwin_save_auto_ok, equalizerwin_save_auto_select);
+				equalizerwin_create_list_window(equalizer_auto_presets, _("Save auto-preset"), &equalizerwin_save_auto_window, GTK_SELECTION_SINGLE, &equalizerwin_save_auto_entry, _("OK"), _("Cancel"), equalizerwin_save_auto_ok, equalizerwin_save_auto_select);
 			else
 				gdk_window_raise(equalizerwin_save_auto_window->window);
 			if ((name = playlist_get_filename(get_playlist_position())) != NULL)
@@ -1331,6 +1527,8 @@ void equalizerwin_load_auto_preset(gchar * filename)
 	if (!cfg.equalizer_autoload)
 		return;
 
+	g_return_if_fail(filename != NULL);
+
 	presetfilename = g_strdup_printf("%s.%s", filename, cfg.eqpreset_extension);
 
 	/*
@@ -1346,6 +1544,7 @@ void equalizerwin_load_auto_preset(gchar * filename)
 	}
 
 	directory = g_dirname(filename);
+	g_free(presetfilename);
 	presetfilename = g_strdup_printf("%s/%s", directory, cfg.eqpreset_default_file);
 	g_free(directory);
 
@@ -1490,7 +1689,7 @@ GtkWidget * equalizerwin_create_conf_window(void)
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), options_vbox, gtk_label_new(_("Options")));
 
         /* 
-         * Ok, Cancel & Apply 
+         * OK, Cancel & Apply 
          */
 
 	hbox = gtk_hbutton_box_new();
@@ -1499,7 +1698,7 @@ GtkWidget * equalizerwin_create_conf_window(void)
 
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-	ok = gtk_button_new_with_label(_("Ok"));
+	ok = gtk_button_new_with_label(_("OK"));
 	gtk_signal_connect(GTK_OBJECT(ok), "clicked", GTK_SIGNAL_FUNC(equalizerwin_conf_ok_cb), NULL);
 	GTK_WIDGET_SET_FLAGS(ok, GTK_CAN_DEFAULT);
 	gtk_box_pack_start(GTK_BOX(hbox), ok, TRUE, TRUE, 0);
